USE omop4;
GO

/* *** BLOCK 1: Extract relevant data element FROM source data and store in #tmp *** */
IF OBJECT_ID('tempdb..#tmp') IS NOT NULL
	DROP TABLE #tmp;
CREATE TABLE #tmp (
	person_id INT
	,eventDate DATE
	,eventConceptId VARCHAR(100)
	,eventType VARCHAR(100)
	,eventNumValue FLOAT NULL
	,eventStringValue VARCHAR(100) NULL
	);
INSERT INTO #tmp
SELECT DISTINCT *
FROM (
	SELECT P0.person_id
		,U.eventDate
		,U.eventConceptId
		,U.eventType
		,U.eventNumValue
		,U.eventStringValue
	FROM omop4.eMergeCohort P0
	LEFT JOIN (
	
		SELECT person_id
		, CONVERT(varchar,year_of_birth)+'-'
						+ CONVERT(varchar, month_of_birth) +'-' 
						+CONVERT(varchar, day_of_birth)  AS eventDate
		,NULL AS eventConceptId
		,'Birthday' AS eventType
		,NULL AS eventNumValue
		,NULL AS eventStringValue
	    FROM omop4.dbo.person
	 
	   UNION ALL
    	SELECT person_id
			,condition_start_date AS eventDate
			,condition_concept_id AS eventConceptId
			,'DxType2Aaa' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.condition_occurrence
		WHERE condition_concept_id IN ('195559','37622385')
		
		UNION ALL
    	SELECT person_id
			,condition_start_date AS eventDate
			,condition_concept_id AS eventConceptId
			,'DxType3Aaa' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.condition_occurrence
		WHERE condition_concept_id IN ('321314','37622384')
		
		UNION ALL
    	SELECT person_id
			,condition_start_date AS eventDate
			,condition_concept_id AS eventConceptId
			,'DxCtrlExcl' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.condition_occurrence
		WHERE condition_concept_id IN ('21001644','195559','321314','37622384','37622385','317585')
		
		UNION ALL
    	SELECT person_id
			,condition_start_date AS eventDate
			,condition_concept_id AS eventConceptId
			,'DxExclType1' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.condition_occurrence
		WHERE condition_concept_id IN ('21001726','79145','200433','201043','258540','315288','321873','378774','432346','436996','441875','443622','321887','35305874','35305940','36717971','36718052','37019347','37622384','37622382','37622385','37622392','37622482')
		
		UNION ALL
    	SELECT person_id
			,condition_start_date AS eventDate
			,condition_concept_id AS eventConceptId
			,'DxType3b' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.condition_occurrence
		WHERE condition_concept_id IN ('21001644','195559','312343','320739','321314','433222','436136','436996','441875','443622','40480639','317585','4256889','37622384','37622385','37622386')
		
		UNION ALL
		SELECT person_id
			,procedure_date AS eventDate
			,procedure_concept_id AS eventConceptId
			,'ProcAaa' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.procedure_occurrence
		WHERE procedure_concept_id in ('2107575','2107574','2107577','2107576','2107573','2107596','2107597','2107595','2107643','2107644','2107631','2107642','2107645','2107646','2107664','2107665')
		
		UNION ALL
	    SELECT person_id
	      ,visit_start_date AS eventDate
	      , NULL AS eventConceptId
	      ,'Visit' AS eventType
	      ,NULL AS eventNumValue
		,NULL AS eventStringValue
	  FROM omop4.dbo.visit_occurrence
	  WHERE DATEDIFF(year, visit_start_date, GETDATE()) <= 5
		
		) U ON P0.person_id = U.person_id
	) Utotal;

/* *** BLOCK 2: Get algorithm related variables *** */
IF OBJECT_ID('tempdb..#AlgVar') IS NOT NULL
	DROP TABLE #AlgVar;
CREATE TABLE #AlgVar (
	person_id INT NOT NULL
	,diseaseName VARCHAR(50) NOT NULL
	,caseControl VARCHAR(50) NOT NULL
	,C00 INT NULL
	,C01 INT NOT NULL
	,C02 INT NOT NULL
	,C03 INT NOT NULL
	,C04 INT NOT NULL
	,C05 INT NOT NULL
	,C06 INT NOT NULL
	,C07 INT NOT NULL
	);
INSERT INTO #AlgVar
SELECT DISTINCT J.person_id
	,'AAA' AS diseaseName
	,'CASE/CONTROL' AS caseControl
	,J0.age AS C00
	,ISNULL(J1.visitCnt, 0) AS C01
	,ISNULL(J2.dxType2AaaCnt, 0) AS C02
	,ISNULL(J3.dxType3AaaCnt, 0) AS C03
	,ISNULL(J4.dxCtrlExclCnt, 0) AS C04
	,ISNULL(J5.dxExclType1Cnt, 0) AS C05
	,ISNULL(J6.dxType3bCnt, 0) AS C06
	,ISNULL(J7.procAaaCnt, 0) AS C07
FROM omop4.eMergeCohort J

LEFT JOIN (
SELECT person_id
, DATEDIFF(year, eventDate, GETDATE()) AS age
FROM #tmp
WHERE eventType = 'Birthday'
) J0
ON J.person_id = J0.person_id

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS visitCnt
	FROM #tmp
	WHERE eventType = 'Visit'
	group by person_id
	) J1 
ON J.person_id = J1.person_id

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS dxType2AaaCnt
	FROM #tmp
	WHERE eventType = 'DxType2Aaa'
	group by person_id
	) J2
ON J.person_id = J2.person_id

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS dxType3AaaCnt
	FROM #tmp
	WHERE eventType = 'DxType3Aaa'
	group by person_id
	) J3 
ON J.person_id = J3.person_id

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS dxCtrlExclCnt
	FROM #tmp
	WHERE eventType = 'DxCtrlExcl'
	group by person_id
	) J4 
ON J.person_id = J4.person_id

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS dxExclType1Cnt
	FROM #tmp
	WHERE eventType = 'DxExclType1'
	group by person_id
	) J5 
ON J.person_id = J5.person_id

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS dxType3bCnt
	FROM #tmp
	WHERE eventType = 'DxType3b'
	group by person_id
	) J6 
ON J.person_id = J6.person_id

LEFT JOIN (
	SELECT person_id
		,count(distinct eventDate) AS procAaaCnt
	FROM #tmp
	WHERE eventType = 'ProcAaa'
	group by person_id
	) J7 
ON J.person_id = J7.person_id
;

/* *** BLOCK 3: Phenotyping **** */
/*
 * @@FOR diagnosing AAA
 * C00.age
 * C01.visitCnt
 * C02.dxType2AaaCnt
 * C03.dxType3AaaCnt
 * C04.dxCtrlExclCnt
 * C05.dxExclType1Cnt
 * C06.dxType3bCnt
 * C07.procAaaCnt
 */
IF OBJECT_ID('tempdb..#phenotype') IS NOT NULL
	DROP TABLE #phenotype;
CREATE TABLE #phenotype (
	person_id INT
	,pCase1 VARCHAR(50)
	,pCase2 VARCHAR(50)
	,pCase3 VARCHAR(50)
	,pCtrl VARCHAR(50)
	);

INSERT INTO #phenotype
SELECT DISTINCT CL.person_id /* CL has duplicate person_id */
	,ISNULL(U1.phenotype, 'NO') AS pCase1
	,ISNULL(U2.phenotype, 'NO') AS pCase2
	,ISNULL(U3.phenotype, 'NO') AS pCase3
	,ISNULL(U4.phenotype, 'NO') AS pCtrl
FROM omop4.eMergeCohort CL
LEFT JOIN (
	SELECT DISTINCT person_id
		,'YES' AS phenotype
	FROM #AlgVar
	WHERE (
			C05=0 AND
			C00 >=40 AND
			C00 <=89 AND
			C07 > 0
			)
	) U1 ON CL.person_id = U1.person_id
LEFT JOIN (
	SELECT DISTINCT person_id
		,'YES' AS phenotype
	FROM #AlgVar
	WHERE (
			C05=0 AND
			C00 >=40 AND
			C00 <=89 AND
			C07 = 0 AND
			C02 > 0
			)
	) U2 ON CL.person_id = U2.person_id
LEFT JOIN (
	SELECT DISTINCT person_id
		,'YES' AS phenotype
	FROM #AlgVar
	WHERE (
			C05=0 AND
			C00 >=40 AND
			C00 <=89 AND
			C07 = 0 AND
			C02 = 0 AND
			C03 > 0 AND
			C01 > 0
			)
	) U3 ON CL.person_id = U3.person_id
LEFT JOIN (
	SELECT DISTINCT person_id
		,'YES' AS phenotype
	FROM #AlgVar
	WHERE (
			C05=0 AND
			C00 >=40 AND
			C00 <=89 AND
			C07 = 0 AND
			C02 = 0 AND
			C03 = 0 AND
			C01 > 0 AND
			C04 = 0
			)
	) U4 ON CL.person_id = U4.person_id;

/* *** For phekb submission *** */
SELECT person_id AS eMERGE_ID
	,CASE WHEN pCase1 = 'YES' THEN 'C49152'
		WHEN pCase2 = 'YES' THEN 'C49152'
		WHEN pCase3 = 'YES' THEN 'C49152'
		WHEN pCtrl = 'YES' THEN 'C28143'
		WHEN pCase1 = 'NO' AND pCase2 ='NO' AND pCase3='NO' AND pCtrl = 'NO' THEN 'NO' END AS Case_Control
FROM #phenotype
ORDER BY eMERGE_ID
;
